@@ -283,3 +283,94 @@ def test_item_mixed_aray():
     t = tomlkit.item(example)
     assert t.as_string().strip() == expected
     assert dumps({"x": {"y": example}}).strip() == "[x]\ny = " + expected
+
+
+def test_build_super_table():
+    doc = tomlkit.document()
+    table = tomlkit.table(True)
+    table.add("bar", {"x": 1})
+    doc.add("foo", table)
+    assert doc.as_string() == "[foo.bar]\nx = 1\n"
+
+
+def test_add_dotted_key():
+    doc = tomlkit.document()
+    doc.add(tomlkit.key(["foo", "bar"]), 1)
+    assert doc.as_string() == "foo.bar = 1\n"
+
+    table = tomlkit.table()
+    table.add(tomlkit.key(["foo", "bar"]), 1)
+    assert table.as_string() == "foo.bar = 1\n"
+
+
+@pytest.mark.parametrize(
+    ("raw", "expected"),
+    [
+        ("true", True),
+        ("false", False),
+    ],
+)
+def test_value_parses_boolean(raw, expected):
+    parsed = tomlkit.value(raw)
+    assert parsed == expected
+
+
+@pytest.mark.parametrize(
+    "raw", ["t", "f", "tru", "fals", "test", "friend", "truthy", "falsify"]
+)
+def test_value_rejects_values_looking_like_bool_at_start(raw):
+    """Reproduces https://github.com/sdispater/tomlkit/issues/165"""
+    with pytest.raises(tomlkit.exceptions.ParseError):
+        tomlkit.value(raw)
+
+
+@pytest.mark.parametrize(
+    "raw",
+    [
+        "truee",
+        "truely",
+        "true-thoughts",
+        "true_hip_hop",
+    ],
+)
+def test_value_rejects_values_having_true_prefix(raw):
+    """Values that have ``true`` or ``false`` as prefix but then have additional chars are rejected."""
+    with pytest.raises(tomlkit.exceptions.ParseError):
+        tomlkit.value(raw)
+
+
+@pytest.mark.parametrize(
+    "raw",
+    [
+        "falsee",
+        "falsely",
+        "false-ideas",
+        "false_prophet",
+    ],
+)
+def test_value_rejects_values_having_false_prefix(raw):
+    """Values that have ``true`` or ``false`` as prefix but then have additional chars are rejected."""
+    with pytest.raises(tomlkit.exceptions.ParseError):
+        tomlkit.value(raw)
+
+
+@pytest.mark.parametrize(
+    "raw",
+    [
+        '"foo"1.2',
+        "truefalse",
+        "1.0false",
+        "100true",
+        "truetrue",
+        "falsefalse",
+        "1.2.3.4",
+        "[][]",
+        "{a=[][]}[]",
+        "true[]",
+        "false{a=1}",
+    ],
+)
+def test_value_rejects_values_with_appendage(raw):
+    """Values that appear valid at the beginning but leave chars unparsed are rejected."""
+    with pytest.raises(tomlkit.exceptions.ParseError):
+        tomlkit.value(raw)
